/*
 *                            COPYRIGHT
 *
 *  cschem - modular/flexible schematics editor - libcschem (core library)
 *  Copyright (C) 2022 Tibor 'Igor2' Palinkas
 *
 *  (Supported by NLnet NGI0 PET Fund in 2022)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Contact:
 *    Project page: http://repo.hu/projects/cschem
 *    lead developer: email to cschem (at) igor2.repo.hu
 *    mailing list: cschem (at) list.repo.hu (send "subscribe")
 */


#include <libcschem/config.h>
#include <libcschem/search.h>
#include <libcschem/util_grp.h>
#include <librnd/core/actions.h>
#include <librnd/core/plugins.h>
#include <sch-rnd/funchash_core.h>
#include <sch-rnd/crosshair.h>

#include "props.h"
#include "propdlg.h"

const char csch_propedit_cookie[] = "propedit";

extern csch_chdr_t *csch_obj_clicked;

int prop_scope_add(csch_propedit_t *pe, const char *cmd, int quiet)
{
	csch_chdr_t *obj = NULL;

	if (strncmp(cmd, "parent", 6) == 0) {
		if (csch_obj_clicked != NULL)
			obj = &csch_obj_clicked->parent->hdr;
		goto add_obj;
	}
	else if (strncmp(cmd, "object", 6) == 0) {
		csch_oidpath_t idp = {0}, *tmp;

		if (cmd[6] == ':') {
			if (csch_oidpath_parse(&idp, cmd+7) != 0) {
				if (!quiet)
					rnd_message(RND_MSG_ERROR, "Failed to convert object ID: '%s'\n", cmd+7);
				return FGW_ERR_ARG_CONV;
			}
			tmp = malloc(sizeof(csch_oidpath_t));
			*tmp = idp;
			csch_oidpath_list_append(&pe->objs, tmp);
		}
		else {
			obj = csch_obj_clicked;

			add_obj:;
			if (obj == NULL) {
				if (!quiet)
					rnd_message(RND_MSG_ERROR, "No object under the cursor\n");
				return FGW_ERR_ARG_CONV;
			}

			tmp = calloc(sizeof(csch_oidpath_t), 1);
			csch_oidpath_from_obj(tmp, obj);
			csch_oidpath_list_append(&pe->objs, tmp);
		}
	}
	else if (strcmp(cmd, "sheet") == 0)
		pe->use_sheet = 1;
	else if (strcmp(cmd, "selection") == 0)
		pe->use_selection = 1;
	else if (strcmp(cmd, "geo") == 0)
		pe->geo = 1;
	else {
		if (!quiet)
			rnd_message(RND_MSG_ERROR, "Invalid propedit scope: %s\n", cmd);
		return FGW_ERR_ARG_CONV;
	}

	return 0;
}

/* pair of prop_scope_add() - uninits the scope */
static void prop_scope_finish(csch_propedit_t *pe)
{
	csch_oidpath_t *idp;

	/* need to remove idpaths from the scope list, else it won't be possible
	   to add them again */
	while((idp = csch_oidpath_list_first(&pe->objs)) != NULL) {
		csch_oidpath_list_remove(idp);
		free(idp);
	}
}

csch_oidpath_t *prop_scope_single_obj(csch_propedit_t *pe)
{
	if (pe->use_selection || pe->use_sheet)
		return NULL;

	if (csch_oidpath_list_length(&pe->objs) != 1)
		return NULL;

	return csch_oidpath_list_first(&pe->objs);
}


static rnd_action_t sch_propedit_action_list[] = {
	{"propedit", csch_act_propedit, csch_acth_propedit, csch_acts_propedit}
/*
	{"propprint", csch_act_propprint, csch_acth_propprint, csch_acts_propprint},
	{"propset", csch_act_propset, csch_acth_propset, csch_acts_propset},
	{"proptoggle", csch_act_proptoggle, csch_acth_proptoggle, csch_acts_proptoggle},
	{"propget", csch_act_propget, csch_acth_propget, csch_acts_propget}
*/
};


int pplg_check_ver_propedit(int ver_needed) { return 0; }

void pplg_uninit_propedit(void)
{
	csch_propdlg_uninit();
	rnd_remove_actions_by_cookie(csch_propedit_cookie);
}

int pplg_init_propedit(void)
{
	RND_API_CHK_VER;

	RND_REGISTER_ACTIONS(sch_propedit_action_list, csch_propedit_cookie);
	csch_propdlg_init();
	return 0;
}

